# -*- coding: utf-8 -*-
"""
Created on Sun Feb 23 2025

@author: Cord Hockemeyer
"""
from kstpy.helpers import kstFrozenset

xpl_basis = {
       kstFrozenset({"a"}),
       kstFrozenset({"b"}),
       kstFrozenset({"a", "c"}),
       kstFrozenset({"b", "c"}),
       kstFrozenset({"a", "b", "d"})
       }

